@@ -29,7 +29,6 @@
 #include "nautilus-directory-notify.h"
 #include "nautilus-directory-private.h"
 #include "nautilus-file-private.h"
-#include "nautilus-autorun.h"
 #include <eel/eel-gtk-macros.h>
 #include <glib/gi18n.h>
 
@@ -691,10 +690,6 @@ vfs_file_poll_for_media (NautilusFile *file)
 static void
 nautilus_vfs_file_init (gpointer object, gpointer klass)
 {
-	NautilusVFSFile *file;
-
-	file = NAUTILUS_VFS_FILE (object);
-
 }
 
 static void
