@@ -18,22 +18,26 @@
  * Boston, MA 02111-1307, USA.
  */
 
+
 #include <config.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
+
+#include "nautilus-pathbar.h"
+
 #include <libnautilus-private/nautilus-file.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-icon-names.h>
 #include <libnautilus-private/nautilus-trash-monitor.h>
-#include <libnautilus-private/nautilus-dnd.h>
 #include <libnautilus-private/nautilus-icon-dnd.h>
-#include "nautilus-pathbar.h"
+
 #include "nautilus-window.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-slot.h"
+#include "nautilus-window-slot-dnd.h"
 
 enum {
         PATH_CLICKED,
@@ -81,39 +85,14 @@ struct _ButtonData
         GtkWidget *image;
         GtkWidget *label;
         guint ignore_changes : 1;
-        guint file_is_hidden : 1;
         guint fake_root : 1;
-
-	NautilusDragSlotProxyInfo drag_info;
 };
 
-G_DEFINE_TYPE (NautilusPathBar,
-	       nautilus_path_bar,
+G_DEFINE_TYPE (NautilusPathBar, nautilus_path_bar,
 	       GTK_TYPE_CONTAINER);
 
-static void     nautilus_path_bar_finalize                 (GObject         *object);
-static void     nautilus_path_bar_dispose                  (GObject         *object);
-static void     nautilus_path_bar_get_preferred_width      (GtkWidget        *widget,
-							    gint             *minimum,
-							    gint             *natural);
-static void     nautilus_path_bar_get_preferred_height     (GtkWidget        *widget,
-							    gint             *minimum,
-							    gint             *natural);
-static void     nautilus_path_bar_unmap                    (GtkWidget       *widget);
-static void     nautilus_path_bar_size_allocate            (GtkWidget       *widget,
-							    GtkAllocation   *allocation);
-static void     nautilus_path_bar_add                      (GtkContainer    *container,
-							    GtkWidget       *widget);
-static void     nautilus_path_bar_remove                   (GtkContainer    *container,
-							    GtkWidget       *widget);
-static void     nautilus_path_bar_forall                   (GtkContainer    *container,
-							    gboolean         include_internals,
-							    GtkCallback      callback,
-							    gpointer         callback_data);
 static void     nautilus_path_bar_scroll_up                (NautilusPathBar *path_bar);
 static void     nautilus_path_bar_scroll_down              (NautilusPathBar *path_bar);
-static gboolean nautilus_path_bar_scroll                   (GtkWidget       *path_bar,
-							    GdkEventScroll  *scroll);
 static void     nautilus_path_bar_stop_scrolling           (NautilusPathBar *path_bar);
 static gboolean nautilus_path_bar_slider_button_press      (GtkWidget       *widget,
 							    GdkEventButton  *event,
@@ -121,14 +100,6 @@ static gboolean nautilus_path_bar_slider_button_press      (GtkWidget       *wid
 static gboolean nautilus_path_bar_slider_button_release    (GtkWidget       *widget,
 							    GdkEventButton  *event,
 							    NautilusPathBar *path_bar);
-static void     nautilus_path_bar_grab_notify              (GtkWidget       *widget,
-							    gboolean         was_grabbed);
-static void     nautilus_path_bar_state_changed            (GtkWidget       *widget,
-							    GtkStateType     previous_state);
-static void     nautilus_path_bar_style_set                (GtkWidget       *widget,
-							    GtkStyle        *previous_style);
-static void     nautilus_path_bar_screen_changed           (GtkWidget       *widget,
-							    GdkScreen       *previous_screen);
 static void     nautilus_path_bar_check_icon_theme         (NautilusPathBar *path_bar);
 static void     nautilus_path_bar_update_button_appearance (ButtonData      *button_data);
 static void     nautilus_path_bar_update_button_state      (ButtonData      *button_data,
@@ -344,57 +315,6 @@ nautilus_path_bar_init (NautilusPathBar *path_bar)
                           path_bar);
 }
 
-static void
-nautilus_path_bar_class_init (NautilusPathBarClass *path_bar_class)
-{
-        GObjectClass *gobject_class;
-        GtkWidgetClass *widget_class;
-        GtkContainerClass *container_class;
-
-        gobject_class = (GObjectClass *) path_bar_class;
-        widget_class = (GtkWidgetClass *) path_bar_class;
-        container_class = (GtkContainerClass *) path_bar_class;
-
-        gobject_class->finalize = nautilus_path_bar_finalize;
-        gobject_class->dispose = nautilus_path_bar_dispose;
-
-	widget_class->get_preferred_height = nautilus_path_bar_get_preferred_height;
-	widget_class->get_preferred_width = nautilus_path_bar_get_preferred_width;
-	widget_class->unmap = nautilus_path_bar_unmap;
-        widget_class->size_allocate = nautilus_path_bar_size_allocate;
-        widget_class->style_set = nautilus_path_bar_style_set;
-        widget_class->screen_changed = nautilus_path_bar_screen_changed;
-        widget_class->grab_notify = nautilus_path_bar_grab_notify;
-        widget_class->state_changed = nautilus_path_bar_state_changed;
-	widget_class->scroll_event = nautilus_path_bar_scroll;
-
-        container_class->add = nautilus_path_bar_add;
-        container_class->forall = nautilus_path_bar_forall;
-        container_class->remove = nautilus_path_bar_remove;
-
-        path_bar_signals [PATH_CLICKED] =
-                g_signal_new ("path-clicked",
-		  G_OBJECT_CLASS_TYPE (path_bar_class),
-		  G_SIGNAL_RUN_FIRST,
-		  G_STRUCT_OFFSET (NautilusPathBarClass, path_clicked),
-		  NULL, NULL,
-		  g_cclosure_marshal_VOID__OBJECT,
-		  G_TYPE_NONE, 1,
-		  G_TYPE_FILE);
-	 path_bar_signals [PATH_SET] =
-		g_signal_new ("path-set",
-		  G_OBJECT_CLASS_TYPE (path_bar_class),
-		  G_SIGNAL_RUN_FIRST,
-		  G_STRUCT_OFFSET (NautilusPathBarClass, path_set),
-		  NULL, NULL,
-		  g_cclosure_marshal_VOID__OBJECT,
-		  G_TYPE_NONE, 1,
-		  G_TYPE_FILE);
-
-	 gtk_container_class_handle_border_width (container_class);
-}
-
-
 static void
 nautilus_path_bar_finalize (GObject *object)
 {
@@ -410,18 +330,9 @@ nautilus_path_bar_finalize (GObject *object)
 	}
 
         g_list_free (path_bar->button_list);
-	if (path_bar->root_path) {
-		g_object_unref (path_bar->root_path);
-		path_bar->root_path = NULL;
-	}
-	if (path_bar->home_path) {
-		g_object_unref (path_bar->home_path);
-		path_bar->home_path = NULL;
-	}
-	if (path_bar->desktop_path) {
-		g_object_unref (path_bar->desktop_path);
-		path_bar->desktop_path = NULL;
-	}
+	g_clear_object (&path_bar->root_path);
+	g_clear_object (&path_bar->home_path);
+	g_clear_object (&path_bar->desktop_path);
 
 	g_signal_handlers_disconnect_by_func (nautilus_trash_monitor_get (),
 					      trash_state_changed_cb, path_bar);
@@ -767,11 +678,9 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
 }
 
 static void
-nautilus_path_bar_style_set (GtkWidget *widget,	GtkStyle  *previous_style)
+nautilus_path_bar_style_updated (GtkWidget *widget)
 {
-        if (GTK_WIDGET_CLASS (nautilus_path_bar_parent_class)->style_set) {
-        	GTK_WIDGET_CLASS (nautilus_path_bar_parent_class)->style_set (widget, previous_style);
-	}
+	GTK_WIDGET_CLASS (nautilus_path_bar_parent_class)->style_updated (widget);
 
         nautilus_path_bar_check_icon_theme (NAUTILUS_PATH_BAR (widget));
 }
@@ -894,6 +803,74 @@ nautilus_path_bar_forall (GtkContainer *container,
 	}
 }
 
+static void
+nautilus_path_bar_grab_notify (GtkWidget *widget,
+			       gboolean   was_grabbed)
+{
+        if (!was_grabbed) {
+                nautilus_path_bar_stop_scrolling (NAUTILUS_PATH_BAR (widget));
+	}
+}
+
+static void
+nautilus_path_bar_state_changed (GtkWidget    *widget,
+			         GtkStateType  previous_state)
+{
+        if (!gtk_widget_get_sensitive (widget)) {
+                nautilus_path_bar_stop_scrolling (NAUTILUS_PATH_BAR (widget));
+	}
+}
+
+static void
+nautilus_path_bar_class_init (NautilusPathBarClass *path_bar_class)
+{
+        GObjectClass *gobject_class;
+        GtkWidgetClass *widget_class;
+        GtkContainerClass *container_class;
+
+        gobject_class = (GObjectClass *) path_bar_class;
+        widget_class = (GtkWidgetClass *) path_bar_class;
+        container_class = (GtkContainerClass *) path_bar_class;
+
+        gobject_class->finalize = nautilus_path_bar_finalize;
+        gobject_class->dispose = nautilus_path_bar_dispose;
+
+	widget_class->get_preferred_height = nautilus_path_bar_get_preferred_height;
+	widget_class->get_preferred_width = nautilus_path_bar_get_preferred_width;
+	widget_class->unmap = nautilus_path_bar_unmap;
+        widget_class->size_allocate = nautilus_path_bar_size_allocate;
+        widget_class->style_updated = nautilus_path_bar_style_updated;
+        widget_class->screen_changed = nautilus_path_bar_screen_changed;
+        widget_class->grab_notify = nautilus_path_bar_grab_notify;
+        widget_class->state_changed = nautilus_path_bar_state_changed;
+	widget_class->scroll_event = nautilus_path_bar_scroll;
+
+        container_class->add = nautilus_path_bar_add;
+        container_class->forall = nautilus_path_bar_forall;
+        container_class->remove = nautilus_path_bar_remove;
+
+        path_bar_signals [PATH_CLICKED] =
+                g_signal_new ("path-clicked",
+		  G_OBJECT_CLASS_TYPE (path_bar_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (NautilusPathBarClass, path_clicked),
+		  NULL, NULL,
+		  g_cclosure_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  G_TYPE_FILE);
+	 path_bar_signals [PATH_SET] =
+		g_signal_new ("path-set",
+		  G_OBJECT_CLASS_TYPE (path_bar_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (NautilusPathBarClass, path_set),
+		  NULL, NULL,
+		  g_cclosure_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  G_TYPE_FILE);
+
+	 gtk_container_class_handle_border_width (container_class);
+}
+
 static void
 nautilus_path_bar_scroll_down (NautilusPathBar *path_bar)
 {
@@ -1076,25 +1053,6 @@ nautilus_path_bar_slider_button_release (GtkWidget      *widget,
         return FALSE;
 }
 
-static void
-nautilus_path_bar_grab_notify (GtkWidget *widget,
-			       gboolean   was_grabbed)
-{
-        if (!was_grabbed) {
-                nautilus_path_bar_stop_scrolling (NAUTILUS_PATH_BAR (widget));
-	}
-}
-
-static void
-nautilus_path_bar_state_changed (GtkWidget    *widget,
-			         GtkStateType  previous_state)
-{
-        if (!gtk_widget_get_sensitive (widget)) {
-                nautilus_path_bar_stop_scrolling (NAUTILUS_PATH_BAR (widget));
-	}
-}
-
-
 
 /* Changes the icons wherever it is needed */
 static void
@@ -1168,7 +1126,6 @@ button_clicked_cb (GtkWidget *button,
         ButtonData *button_data;
         NautilusPathBar *path_bar;
         GList *button_list;
-        gboolean child_is_hidden;
 
         button_data = BUTTON_DATA (data);
         if (button_data->ignore_changes) {
@@ -1182,54 +1139,9 @@ button_clicked_cb (GtkWidget *button,
 
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
 
-        if (button_list->prev) {
-                ButtonData *child_data;
-
-                child_data = BUTTON_DATA (button_list->prev->data);
-                child_is_hidden = child_data->file_is_hidden;
-        } else {
-                child_is_hidden = FALSE;
-	}
         g_signal_emit (path_bar, path_bar_signals [PATH_CLICKED], 0, button_data->path);
 }
 
-static NautilusIconInfo *
-get_custom_user_icon_info (ButtonData *button_data)
-{
-/* Bug 80925: With tiny display sizes we get huge memory allocations. */
-#if 0
-	NautilusIconInfo *icon_info;
-	GFile *icon_file;
-	GIcon *icon;
-	char *custom_icon_uri;
-
-	icon = NULL;
-
-	if (button_data->file != NULL) {
-		custom_icon_uri = nautilus_file_get_custom_icon (button_data->file);
-		if (custom_icon_uri != NULL) {
-			icon_file = g_file_new_for_uri (custom_icon_uri);
-
-			if (g_file_is_native (icon_file)) {
-				icon = g_file_icon_new (icon_file);
-			}
-
-			g_object_unref (icon_file);
-			g_free (custom_icon_uri);
-		}
-	}
-
-	if (icon != NULL) {
-		icon_info = nautilus_icon_info_lookup (icon, NAUTILUS_PATH_BAR_ICON_SIZE);
-		g_object_unref (icon);
-
-		return icon_info;
-	}
-#endif
-
-	return NULL;
-}
-
 static NautilusIconInfo *
 get_type_icon_info (ButtonData *button_data)
 {
@@ -1276,9 +1188,6 @@ button_data_free (ButtonData *button_data)
 		nautilus_file_unref (button_data->file);
 	}
 
-	g_object_unref (button_data->drag_info.target_location);
-	button_data->drag_info.target_location = NULL;
-
         g_free (button_data);
 }
 
@@ -1287,6 +1196,8 @@ get_dir_name (ButtonData *button_data)
 {
 	if (button_data->type == DESKTOP_BUTTON) {
 		return _("Desktop");
+	} else if (button_data->type == HOME_BUTTON) {
+		return _("Home");
 	} else {
 		return button_data->dir_name;
 	}
@@ -1344,10 +1255,7 @@ nautilus_path_bar_update_button_appearance (ButtonData *button_data)
 			gtk_image_set_from_pixbuf (GTK_IMAGE (button_data->image), button_data->custom_icon);  
 			gtk_widget_show (GTK_WIDGET (button_data->image));
 		} else {
-			icon_info = get_custom_user_icon_info (button_data);
-			if (icon_info == NULL) {
-				icon_info = get_type_icon_info (button_data);
-			}
+			icon_info = get_type_icon_info (button_data);
 
 			pixbuf = NULL;
 
@@ -1601,17 +1509,25 @@ button_data_file_changed (NautilusFile *file,
 	} else if (nautilus_file_is_gone (file)) {
 		gint idx, position;
 
-		/* remove this and the following buttons */
-		position = g_list_position (path_bar->button_list,
-					    g_list_find (path_bar->button_list, button_data));
+		/* if the current or a parent location are gone, don't do anything, as the view
+		 * will get the event too and call us back.
+		 */
+		current_location = nautilus_file_get_location (current_button_data->file);
 
-		if (position != -1) {
-			for (idx = 0; idx <= position; idx++) {
-				gtk_container_remove (GTK_CONTAINER (path_bar),
-						      BUTTON_DATA (path_bar->button_list->data)->button);
+		if (g_file_has_prefix (location, current_location)) {
+			/* remove this and the following buttons */
+			position = g_list_position (path_bar->button_list,
+						    g_list_find (path_bar->button_list, button_data));
+
+			if (position != -1) {
+				for (idx = 0; idx <= position; idx++) {
+					gtk_container_remove (GTK_CONTAINER (path_bar),
+							      BUTTON_DATA (path_bar->button_list->data)->button);
+				}
 			}
 		}
 
+		g_object_unref (current_location);
 		g_object_unref (location);
 		return;
 	}
@@ -1634,8 +1550,7 @@ static ButtonData *
 make_directory_button (NautilusPathBar  *path_bar,
 		       NautilusFile     *file,
 		       gboolean          current_dir,
-		       gboolean          base_dir,	
-		       gboolean          file_is_hidden)
+		       gboolean          base_dir)
 {
 	GFile *path;
         GtkWidget *child;
@@ -1647,7 +1562,6 @@ make_directory_button (NautilusPathBar  *path_bar,
 	child = NULL;
 	label_alignment = NULL;
 
-        file_is_hidden = !! file_is_hidden;
         /* Is it a special button? */
         button_data = g_new0 (ButtonData, 1);
 
@@ -1656,8 +1570,6 @@ make_directory_button (NautilusPathBar  *path_bar,
 	gtk_button_set_focus_on_click (GTK_BUTTON (button_data->button), FALSE);
 	/* TODO update button type when xdg directories change */
 
-	button_data->drag_info.target_location = g_object_ref (path);
-
 	button_data->image = gtk_image_new ();
 
         switch (button_data->type) {
@@ -1703,8 +1615,6 @@ make_directory_button (NautilusPathBar  *path_bar,
 					  button_data);
 	}
 
-        button_data->file_is_hidden = file_is_hidden;
-
 	/* FIXME: Maybe we dont need this alignment at all and we can
 	 * use GtkMisc aligments or even GtkWidget:halign/valign center.
 	 *
@@ -1725,8 +1635,7 @@ make_directory_button (NautilusPathBar  *path_bar,
 
 	setup_button_drag_source (button_data);
 
-	nautilus_drag_slot_proxy_init (button_data->button,
-				       &(button_data->drag_info));
+	nautilus_drag_slot_proxy_init (button_data->button, path, NULL);
 
 	g_object_unref (path);
 
@@ -1825,7 +1734,7 @@ nautilus_path_bar_update_path (NautilusPathBar *path_bar,
         while (file != NULL) {
 		parent_file = nautilus_file_get_parent (file);
 		last_directory = !parent_file;
-		button_data = make_directory_button (path_bar, file, first_directory, last_directory, FALSE);
+		button_data = make_directory_button (path_bar, file, first_directory, last_directory);
 		nautilus_file_unref (file);
 
 		if (first_directory) {
@@ -1910,57 +1819,3 @@ nautilus_path_bar_get_path_for_button (NautilusPathBar *path_bar,
 
 	return NULL;
 }
-
-/**
- * _nautilus_path_bar_up:
- * @path_bar: a #NautilusPathBar
- * 
- * If the selected button in the pathbar is not the furthest button "up" (in the
- * root direction), act as if the user clicked on the next button up.
- **/
-void
-nautilus_path_bar_up (NautilusPathBar *path_bar)
-{
-        GList *l;
-
-        for (l = path_bar->button_list; l; l = l->next) {
-                GtkWidget *button = BUTTON_DATA (l->data)->button;
-                if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
-			if (l->next) {
-			        GtkWidget *next_button = BUTTON_DATA (l->next->data)->button;
-	      			button_clicked_cb (next_button, l->next->data);
-			}
-	  		break;
-		}
-        }
-}
-
-/**
- * _nautilus_path_bar_down:
- * @path_bar: a #NautilusPathBar
- * 
- * If the selected button in the pathbar is not the furthest button "down" (in the
- * leaf direction), act as if the user clicked on the next button down.
- **/
-void
-nautilus_path_bar_down (NautilusPathBar *path_bar)
-{
-        GList *l;
-
-        for (l = path_bar->button_list; l; l = l->next) {
-                GtkWidget *button = BUTTON_DATA (l->data)->button;
-                if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button))) {
-	  		if (l->prev) {
-		      		GtkWidget *prev_button = BUTTON_DATA (l->prev->data)->button;
-	      			button_clicked_cb (prev_button, l->prev->data);
-	    		}
-	  		break;
-		}
-        }
-}
-
-GtkWidget *
-nautilus_path_bar_get_button_from_button_list_entry (gpointer entry)
-{
-  return BUTTON_DATA(entry)->button;
-}
